package net.ibizsys.central.plugin.boot.core.domain;

import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.util.TypeUtils;
import com.fasterxml.jackson.annotation.JsonAnyGetter;
import com.fasterxml.jackson.annotation.JsonAnySetter;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.*;
import lombok.extern.slf4j.Slf4j;
import net.ibizsys.central.dataentity.IDataEntityRuntime;
import net.ibizsys.central.dataentity.service.IDEMethodDTO;
import net.ibizsys.central.dataentity.service.IDEMethodDTORuntime;
import net.ibizsys.central.plugin.boot.core.helper.FieldCache;
import net.ibizsys.central.util.IEntityDTO;
import net.ibizsys.model.dataentity.defield.IPSDEField;
import net.ibizsys.model.dataentity.service.IPSDEMethodDTOField;
import net.ibizsys.runtime.dataentity.DataEntityRuntimeException;
import net.ibizsys.runtime.dataentity.service.DEMethodDTOFieldTypes;
import net.ibizsys.runtime.dataentity.service.DEMethodDTOTypes;
import net.ibizsys.runtime.util.DataTypeUtils;
import net.ibizsys.runtime.util.DataTypes;
import net.ibizsys.runtime.util.IEntity;
import org.springframework.cglib.beans.BeanMap;
import org.springframework.data.annotation.Transient;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Timestamp;
import java.util.*;

@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@Slf4j
public class BaseData  implements IEntityDTO  {

    @JsonIgnore
    @JSONField(serialize = false)
    private transient IDEMethodDTORuntime iDEMethodDTORuntime = null;

    @JsonIgnore
    @JSONField(serialize = false)
    private boolean actionInputDTO = false;

    public void setActionInputDTO(Boolean actionInputDTO)
    {
        this.actionInputDTO=actionInputDTO.booleanValue();
    }

    @JsonIgnore
    @JSONField(serialize = false)
    private boolean enableAny = false;

    public void setEnableAny(Boolean enableAny)
    {
        this.actionInputDTO=enableAny.booleanValue();
    }

    @Override
    public void init(IDEMethodDTORuntime iDEMethodDTORuntime, Object dtoData, boolean bDTOData, boolean bIn) throws Exception {
        this.iDEMethodDTORuntime = iDEMethodDTORuntime;
        if(this.getDEMethodDTORuntime() == null) {
            throw new Exception("传入实体方法DTO运行时对象无效");
        }
        enableAny = this.getDEMethodDTORuntime().isEnableAny();
        if(DEMethodDTOTypes.DEACTIONINPUT.equals(this.getDEMethodDTORuntime().getPSDEMethodDTO().getType())) {
            actionInputDTO = true;
        }
        reload(dtoData, bDTOData, bIn);
        onInit();
    }

    protected void onInit() throws Exception {

    }

    @Override
    public void init(IDEMethodDTORuntime iDEMethodDTORuntime, Object dtoData, boolean bDTOData) throws Exception {
        init(iDEMethodDTORuntime, dtoData, bDTOData, false);
    }

    @Override
    @JsonIgnore
    @JSONField(serialize = false)
    public IDEMethodDTORuntime getDEMethodDTORuntime() {
        return this.iDEMethodDTORuntime;
    }

    @Override
    public void setDEMethodDTORuntime(IDEMethodDTORuntime iDEMethodDTORuntime) {
        this.iDEMethodDTORuntime = iDEMethodDTORuntime;
        this.enableAny = false;
        this.actionInputDTO = false;
        if(this.getDEMethodDTORuntime() != null) {
            enableAny = this.getDEMethodDTORuntime().isEnableAny();
            if(DEMethodDTOTypes.DEACTIONINPUT.equals(this.getDEMethodDTORuntime().getPSDEMethodDTO().getType())) {
                actionInputDTO = true;
            }
        }
    }

    @JsonIgnore
    @JSONField(serialize = false)
    private Set<String> focusNull=new HashSet<>();

    public void setFocusNull(Set<String> focusNull)
    {
        if(focusNull==null)
            this.focusNull.clear();
        else
            this.focusNull=focusNull;
    }

    public void modify(String field,Object val) {
        if(val==null) {
            focusNull.add(field);
        }
        else {
            focusNull.remove(field);
        }
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public Set<String> getFocusNull() {
        if(focusNull.size()>0 && paramMap.containsKey("dirtyflagenable"))
        {
            Set<String> nocheck=new HashSet<>();
            for(String key:focusNull)
            {
                if(!paramMap.containsKey(key+"dirtyflag")) {
                    nocheck.add(key);
                }
            }
            for(String key:nocheck) {
                focusNull.remove(key);
            }
        }
        return focusNull;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    private Map<String,Object> paramMap=new HashMap<String,Object>();

    public void setParamMap(Map<String,Object> paramMap)
    {
        if(paramMap==null)
            this.paramMap.clear();
        else
            this.paramMap=paramMap;
    }

    @JsonAnyGetter
    @JSONField(name = "_any", unwrapped = true, serialize = true, deserialize = false)
    public Map<String , Object> anyObject() {
        return paramMap;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    private Map<String , Object> _allParamMap=null;

    @JsonIgnore
    @JSONField(serialize = false)
    public Map<String , Object> any() {
        if(_allParamMap==null)
            _allParamMap = new HashMap<>();
        _allParamMap.clear();
        _allParamMap.putAll(this.paramMap);
        _allParamMap.putAll(this.getMap());
        focusNull.forEach(item->_allParamMap.put(item,null));
        return _allParamMap;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    @Transient
    private BeanMap map;

    @JsonIgnore
    @JSONField(serialize = false)
    @Transient
    private BeanMap getMap()
    {
        if(map==null) {
            map=BeanMap.create(this);
        }
        return  map;
    }


    @Override
    public Object get(String strName) {
        if(this.getDEMethodDTORuntime() != null) {
            if(isActionInputDTO()) {
                IPSDEMethodDTOField iPSDEMethodDTOField = this.getDEMethodDTORuntime().getPSDEMethodDTOField(strName, true);
                if(iPSDEMethodDTOField == null) {
                    if(this.isEnableAny()) {
                        return getObject(strName.toLowerCase());
                    }
                    return null;
                }
                return getObject(iPSDEMethodDTOField.getLowerCaseName());
            }
            else{
                IPSDEMethodDTOField iPSDEMethodDTOField = this.getDEMethodDTORuntime().getPSDEMethodDTOFieldByDEField(strName, true);
                if(iPSDEMethodDTOField == null) {
                    if(this.isEnableAny()) {
                        return getObject(strName.toLowerCase());
                    }
                    return null;
                }
                return getObject(iPSDEMethodDTOField.getLowerCaseName());
            }
        }
        else {
            return getObject(strName);
        }
    }

    public Object getObject(String strName) {
        String fieldRealName= FieldCache.getFieldRealName(this.getClass(),strName);
        if(!StringUtils.isEmpty(fieldRealName)) {
            return getMap().get(fieldRealName);
        }
        else {
            return this.paramMap.get(strName);
        }
    }

    public boolean contains(String strName) {
        if(this.getDEMethodDTORuntime() != null) {
            if(isActionInputDTO()) {
                IPSDEMethodDTOField iPSDEMethodDTOField = this.getDEMethodDTORuntime().getPSDEMethodDTOField(strName, true);
                if(iPSDEMethodDTOField == null) {
                    if(this.isEnableAny()) {
                        return containsKey(strName.toLowerCase());
                    }
                    return false;
                }
                return containsKey(iPSDEMethodDTOField.getLowerCaseName());
            }
            else {
                IPSDEMethodDTOField iPSDEMethodDTOField = this.getDEMethodDTORuntime().getPSDEMethodDTOFieldByDEField(strName, true);
                if(iPSDEMethodDTOField == null) {
                    if(this.isEnableAny()) {
                        return containsKey(strName.toLowerCase());
                    }
                    return false;
                }
                return containsKey(iPSDEMethodDTOField.getLowerCaseName());
            }
        }
        else {
            return containsKey(strName);
        }
    }

    public boolean containsKey(String strName) {
        if(getFocusNull().contains(strName))
            return true;
        String fieldRealName= FieldCache.getFieldRealName(this.getClass(),strName);
        if(!StringUtils.isEmpty(fieldRealName)) {
            if(getFocusNull().contains(strName))
                return true;

            return getMap().get(fieldRealName)!=null;
        }
        else {
            return this.paramMap.containsKey(strName);
        }
    }

    public void set(String strName, String strValue) {
        if(this.getDEMethodDTORuntime() != null) {
            IPSDEMethodDTOField iPSDEMethodDTOField = null;
            if(isActionInputDTO()) {
                iPSDEMethodDTOField = this.getDEMethodDTORuntime().getPSDEMethodDTOField(strName, true);
            }
            else{
                iPSDEMethodDTOField = this.getDEMethodDTORuntime().getPSDEMethodDTOFieldByDEField(strName, true);
            }
            if(iPSDEMethodDTOField != null) {
                int nStdDataType = iPSDEMethodDTOField.getStdDataType();
                if(nStdDataType != DataTypes.UNKNOWN && !DataTypeUtils.isStringDataType(nStdDataType)) {
                    Object objValue = null;
                    try {
                        objValue = this.getDEMethodDTORuntime().getSystemRuntime().convertValue(nStdDataType, strValue);
                    } catch (Exception ex) {
                        throw new DataEntityRuntimeException(this.getDEMethodDTORuntime().getDataEntityRuntime()
                                ,String.format("转化数据[%1$s]至[%2$s]发生异常，%3$s", strValue, DataTypeUtils.getTypeName(nStdDataType), ex.getMessage())
                                ,ex);
                    }
                    set(strName, objValue);
                    return ;
                }
            }

        }
        set(strName, (Object)strValue);
    }

    public void set(String strName, Object objValue) {
        if(this.getDEMethodDTORuntime() != null) {
            if(isActionInputDTO()) {
                IPSDEMethodDTOField iPSDEMethodDTOField = this.getDEMethodDTORuntime().getPSDEMethodDTOField(strName, true);
                if(iPSDEMethodDTOField == null) {
                    if(this.isEnableAny()) {
                        setObject(strName.toLowerCase(), objValue);
                    }
                    else {
                        log.warn(String.format("DTO对象[%1$s]不支持属性[%2$s]，无法设置值", this.getDEMethodDTORuntime().getId(), strName));
                    }
                }
                else
                    setObject(iPSDEMethodDTOField.getLowerCaseName(), objValue);
            }
            else{
                IPSDEMethodDTOField iPSDEMethodDTOField = this.getDEMethodDTORuntime().getPSDEMethodDTOFieldByDEField(strName, true);
                if(iPSDEMethodDTOField == null) {
                    if(this.isEnableAny()) {
                        setObject(strName.toLowerCase(), objValue);
                    }
                    else {
                        log.warn(String.format("DTO对象[%1$s]不支持属性[%2$s]，无法设置值", this.getDEMethodDTORuntime().getId(), strName));
                    }
                }
                else
                    setObject(iPSDEMethodDTOField.getLowerCaseName(), objValue);
            }
        }
        else {
            setObject(strName, objValue);
        }
    }

    @JsonAnySetter
    @JSONField(name = "_any", unwrapped = true, serialize = false, deserialize = true)
    public void setObject(String field, Object value) {
        String fieldRealName=FieldCache.getFieldRealName(this.getClass(),field);
        if(!StringUtils.isEmpty(fieldRealName)) {
            if (value == null) {
                getMap().put(fieldRealName, null);
            }
            else {
                getMap().put(fieldRealName, FieldCache.fieldValueOf(this.getClass(), fieldRealName, value));
            }
        }
        else {
            this.paramMap.put(field,value);
        }
    }

    public void reset(String strName) {
        if(this.getDEMethodDTORuntime() != null) {
            if(isActionInputDTO()) {
                IPSDEMethodDTOField iPSDEMethodDTOField = this.getDEMethodDTORuntime().getPSDEMethodDTOField(strName, true);
                if(iPSDEMethodDTOField == null) {
                    if(this.isEnableAny()) {
                        resetObject(strName.toLowerCase());
                    }
                    else {
                        log.warn(String.format("DTO对象[%1$s]不支持属性[%2$s]，无法重置值", this.getDEMethodDTORuntime().getId(), strName));
                    }
                }
                else
                    resetObject(iPSDEMethodDTOField.getLowerCaseName());
            }
            else {
                IPSDEMethodDTOField iPSDEMethodDTOField = this.getDEMethodDTORuntime().getPSDEMethodDTOFieldByDEField(strName, true);
                if(iPSDEMethodDTOField == null) {
                    if(this.isEnableAny()) {
                        resetObject(strName.toLowerCase());
                    }
                    else {
                        log.warn(String.format("DTO对象[%1$s]不支持属性[%2$s]，无法重置值", this.getDEMethodDTORuntime().getId(), strName));
                    }
                }
                else
                    resetObject(iPSDEMethodDTOField.getLowerCaseName());
            }
        }
        else {
            resetObject(strName);
        }
    }

    public void resetObject(String strName) {
        String fieldRealName= FieldCache.getFieldRealName(this.getClass(),strName);
        if(!StringUtils.isEmpty(fieldRealName)) {
            getMap().put(fieldRealName,null);
            focusNull.remove(fieldRealName);
        }
        else {
            this.paramMap.remove(strName);
        }

    }

    @JsonIgnore
    @JSONField(serialize = false)
    public void resetAll() {
        getMap().keySet().forEach(key->{
            if(!key.equals("map"))
                map.put(key,null);
        });
        this.paramMap.clear();
        this.focusNull.clear();
    }



    public void copyTo(IEntity iEntity) {
        copyTo(iEntity, false);
    }

    @Override
    public void copyTo(IEntity iEntity, boolean bDTOData) {
        Assert.notNull(iEntity, "传入目标对象无效");
        Map<String,Object> map=toMap();
        if(!bDTOData && getDEMethodDTORuntime() != null) {
            if(map == null || map.size() == 0) {
                return ;
            }
            List<IPSDEMethodDTOField> psDEMethodDTOFieldList = getDEMethodDTORuntime().getPSDEMethodDTO().getPSDEMethodDTOFields();
            if(psDEMethodDTOFieldList != null) {
                for(IPSDEMethodDTOField iPSDEMethodDTOField : psDEMethodDTOFieldList) {
                    String strFieldName = iPSDEMethodDTOField.getLowerCaseName();
                    if(!map.containsKey(strFieldName)) {
                        continue;
                    }

                    Object objValue = map.remove(strFieldName);
                    if(iPSDEMethodDTOField.getPSDEField() != null) {
                        iEntity.set(iPSDEMethodDTOField.getPSDEField().getLowerCaseName(), objValue);
                    }
                    else {
                        iEntity.set(strFieldName, objValue);
                    }
                }
            }
        }
        for(java.util.Map.Entry<String, Object> entry : map.entrySet()) {
            iEntity.set(entry.getKey(), entry.getValue());
        }
    }



    public void copyTo(Map<String, Object> destMap) {
        this.copyTo(destMap, false);
    }

    @Override
    public void copyTo(Map<String, Object> destMap, boolean bDTOData) {
        Assert.notNull(destMap, "传入目标对象无效");
        Map<String,Object> map=toMap();
        if(!bDTOData && getDEMethodDTORuntime() != null) {
            if(map == null || map.size() == 0) {
                return ;
            }

            List<IPSDEMethodDTOField> psDEMethodDTOFieldList = getDEMethodDTORuntime().getPSDEMethodDTO().getPSDEMethodDTOFields();
            if(psDEMethodDTOFieldList != null) {
                for(IPSDEMethodDTOField iPSDEMethodDTOField : psDEMethodDTOFieldList) {
                    String strFieldName = iPSDEMethodDTOField.getLowerCaseName();
                    if(!map.containsKey(strFieldName)) {
                        continue;
                    }

                    Object objValue = map.remove(strFieldName);
                    if(iPSDEMethodDTOField.getPSDEField() != null) {
                        destMap.put(iPSDEMethodDTOField.getPSDEField().getLowerCaseName(), objValue);
                    }
                    else {
                        destMap.put(strFieldName, objValue);
                    }
                }
            }

            for(java.util.Map.Entry<String, Object> entry : map.entrySet()) {
                destMap.put(entry.getKey(), entry.getValue());
            }
        }
        else {
            destMap.putAll(map);
        }
    }


    @Override
    public void reload(Object objData, boolean bDTOData, boolean bIn) throws Exception {
        if(objData == null) {
            this.resetAll();
            return;
        }
        if(bDTOData) {
            //DTO 数据为直接缓存
            if (objData instanceof IEntityDTO) {
                //需要判断DTO是否一致
                IEntityDTO srcEntityDTO = (IEntityDTO)objData;
                if(srcEntityDTO.getDEMethodDTORuntime()!=null && this.getDEMethodDTORuntime() !=null && this.getDEMethodDTORuntime().getId().equals(srcEntityDTO.getDEMethodDTORuntime().getId())) {
                    ((IEntityDTO) objData).copyTo(this,true);
                }
                else {
                    //不一致，使用复制方式
                    this.resetAll();
                    if(getDEMethodDTORuntime()!=null) {
                        List<IPSDEMethodDTOField> psDEMethodDTOFieldList = getDEMethodDTORuntime().getPSDEMethodDTO().getPSDEMethodDTOFields();
                        if(psDEMethodDTOFieldList != null) {
                            for(IPSDEMethodDTOField iPSDEMethodDTOField : psDEMethodDTOFieldList) {

                                Object objValue = null;
                                IPSDEField iPSDEField = iPSDEMethodDTOField.getPSDEField();
                                if(iPSDEField != null){
                                    if(srcEntityDTO.contains(iPSDEField.getLowerCaseName())) {
                                        objValue = srcEntityDTO.get(iPSDEField.getLowerCaseName());
                                    }
                                    else {
                                        continue;
                                    }
                                }
                                else {
                                    if(srcEntityDTO.contains(iPSDEMethodDTOField.getLowerCaseName())) {
                                        objValue = srcEntityDTO.get(iPSDEMethodDTOField.getLowerCaseName());
                                    }
                                    else {
                                        continue;
                                    }
                                }
                                setObject(iPSDEMethodDTOField.getLowerCaseName(), objValue);
                            }
                        }

                        //复制非DTO属性
                        if(srcEntityDTO.any()!=null) {
                            for(java.util.Map.Entry<String ,Object> entry:srcEntityDTO.any().entrySet()) {
                                String strName = entry.getKey().toLowerCase();
                                if(!this.containsKey(strName)) {
                                    setObject(strName, entry.getValue());
                                }
                            }
                        }
                    }
                    else {
                        ((IEntityDTO) objData).copyTo(this,true);
                    }
                }
            }
            else {
                Map<String,Object> srcmap = null;
                if (objData instanceof Map) {
                    srcmap = (Map<String,Object>)objData;
                }
                else
                if (objData instanceof IEntity) {
                    srcmap = ((IEntity)objData).any();
                }
                else {
                    throw new Exception("无法识别的数据对象");
                }

                Map<String, Object> map = new HashMap<>();
                map.putAll(srcmap);


                //对成员数据进行转化
                if(getDEMethodDTORuntime() != null) {
                    List<IPSDEMethodDTOField> psDEMethodDTOFieldList = getDEMethodDTORuntime().getPSDEMethodDTO().getPSDEMethodDTOFields();
                    if(psDEMethodDTOFieldList != null) {
                        for(IPSDEMethodDTOField iPSDEMethodDTOField : psDEMethodDTOFieldList) {
                            Object dtoData = map.get(iPSDEMethodDTOField.getLowerCaseName());
                            if(DEMethodDTOFieldTypes.DTO.equals(iPSDEMethodDTOField.getType())
                                    ||DEMethodDTOFieldTypes.DTOS.equals(iPSDEMethodDTOField.getType())) {
                                if(dtoData != null) {
                                    IDataEntityRuntime refDataEntityRuntime = this.getDEMethodDTORuntime().getSystemRuntime().getDataEntityRuntime(iPSDEMethodDTOField.getRefPSDataEntityMust().getId());
                                    if(DEMethodDTOFieldTypes.DTOS.equals(iPSDEMethodDTOField.getType())) {
                                        //列表模式
                                        if(!(dtoData instanceof List)) {
                                            throw new DataEntityRuntimeException(this.getDEMethodDTORuntime().getDataEntityRuntime(), this.getDEMethodDTORuntime(), String.format("属性[%1$s]传入数据类型不正确", iPSDEMethodDTOField.getName()));
                                        }
                                        List list = (List)dtoData;
                                        List dtoList=  new ArrayList();
                                        for(Object item : list) {
                                            IDEMethodDTO iDEMethodDTO = refDataEntityRuntime.getDEMethodDTO(iPSDEMethodDTOField.getRefPSDEMethodDTOMust(), item);
                                            dtoList.add(iDEMethodDTO);

                                        }
                                        map.put(iPSDEMethodDTOField.getLowerCaseName(), dtoList);
                                    }
                                    else {
                                        IDEMethodDTO iDEMethodDTO = refDataEntityRuntime.getDEMethodDTO(iPSDEMethodDTOField.getRefPSDEMethodDTOMust(), dtoData);
                                        map.put(iPSDEMethodDTOField.getLowerCaseName(), iDEMethodDTO);
                                    }
                                }
                                continue;
                            }
                            else {
                                if(dtoData != null && dtoData instanceof String) {
                                    int nStdDataType = iPSDEMethodDTOField.getStdDataType();
                                    if(nStdDataType != DataTypes.UNKNOWN && !DataTypeUtils.isStringDataType(nStdDataType)) {
                                        Object objValue = null;
                                        try {
                                            objValue = this.getDEMethodDTORuntime().getSystemRuntime().convertValue(nStdDataType, (String)dtoData);
                                        } catch (Exception ex) {
                                            throw new DataEntityRuntimeException(this.getDEMethodDTORuntime().getDataEntityRuntime()
                                                    ,String.format("转化数据[%1$s]至[%2$s]发生异常，%3$s", dtoData, DataTypeUtils.getTypeName(nStdDataType), ex.getMessage())
                                                    ,ex);
                                        }
                                        map.put(iPSDEMethodDTOField.getLowerCaseName(), objValue);
                                    }
                                }
                                continue;
                            }
                        }
                    }
                }


                setAny(map);
            }

        }
        else {
            this.resetAll();
            //需要进行转化
            Map<String,Object> srcmap = null;
            if (objData instanceof Map) {
                srcmap = (Map<String,Object>)objData;
            }
            else
            if (objData instanceof IEntity) {
                srcmap = ((IEntity)objData).any();
            }
            else {
                throw new Exception("无法识别的数据对象");
            }

            Map<String, Object> map = new HashMap<>();
            map.putAll(srcmap);

            if (map!=null) {
                if(getDEMethodDTORuntime()!=null) {
                    List<IPSDEMethodDTOField> psDEMethodDTOFieldList = getDEMethodDTORuntime().getPSDEMethodDTO().getPSDEMethodDTOFields();
                    if(psDEMethodDTOFieldList != null) {
                        for(IPSDEMethodDTOField iPSDEMethodDTOField : psDEMethodDTOFieldList) {
                            Object objValue = null;
                            IPSDEField iPSDEField = iPSDEMethodDTOField.getPSDEField();
                            if(iPSDEField != null){
                                if(map.containsKey(iPSDEField.getLowerCaseName())) {
                                    objValue = map.remove(iPSDEField.getLowerCaseName());
                                }
                                else
                                if(map.containsKey(iPSDEField.getName())) {
                                    objValue = map.remove(iPSDEField.getName());
                                }
                                else {
                                    continue;
                                }
                            }
                            else {
                                if(map.containsKey(iPSDEMethodDTOField.getLowerCaseName())) {
                                    objValue = map.remove(iPSDEMethodDTOField.getLowerCaseName());
                                }
                                else {
                                    continue;
                                }
                            }
                            setObject(iPSDEMethodDTOField.getLowerCaseName(), objValue);
                        }
                    }

                    //复制非DTO属性
                    for(java.util.Map.Entry<String ,Object> entry:map.entrySet()) {
                        String strName = entry.getKey().toLowerCase();
                        if(!containsKey(strName)) {
                            setObject(strName, entry.getValue());
                        }
                    }
                }
                else {
                    setAny(map);
                }

            }

        }

        if(bIn) {
            this.resetReadOnlyFields();
        }
    }

    @Override
    public void reload(Object objData, boolean bDTOData) throws Exception {
        reload(objData, bDTOData, false);
    }


    @JsonIgnore
    @JSONField(serialize = false)
    @Override
    public void resetReadOnlyFields() {
        if(getDEMethodDTORuntime()!=null) {
            List<IPSDEMethodDTOField> psDEMethodDTOFieldList = getDEMethodDTORuntime().getPSDEMethodDTO().getPSDEMethodDTOFields();
            if(psDEMethodDTOFieldList != null) {
                for(IPSDEMethodDTOField iPSDEMethodDTOField : psDEMethodDTOFieldList) {
                    if(iPSDEMethodDTOField.isReadOnly()) {
                        resetObject(iPSDEMethodDTOField.getLowerCaseName());
                    }
                }
            }
        }
    }


    @JsonIgnore
    protected void setAny(Map<String, Object> map){
        this.setAny(map, false);
    }

    @JsonIgnore
    protected void setAny(Map<String, Object> map, boolean bCopyMode){
        {
            for(java.util.Map.Entry<String, Object> entry : map.entrySet()) {
                this.setObject(entry.getKey(), entry.getValue());
            }
        }
    }


    public IEntityDTO put(String strName, Object objValue) {
        this.set(strName,objValue);
        return this;
    }

    @Override
    public IEntityDTO put(String strName, String strValue) {
        this.set(strName,strValue);
        return this;
    }


    public void putAll(Map<String, Object> map) {
        for(java.util.Map.Entry<String, Object> entry : map.entrySet()) {
            this.set(entry.getKey(), entry.getValue());
        }
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public Map<String, Object> toMap()
    {
        Map<String, Object> map = new HashMap<>();
        map.putAll(this.paramMap);
        map.putAll(this.getMap());
        focusNull.forEach(item->map.put(item,null));
        return map;

    }



    @Override
    public Timestamp getTimestamp(String strName, Timestamp def) {
        try {
            return get(strName,Timestamp.class,def);
        }
        catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }


    @Override
    public BigDecimal getBigDecimal(String strName, BigDecimal fDefault) {
        try {
            return get(strName,BigDecimal.class,fDefault);
        }
        catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }


    @Override
    public Integer getInteger(String strName, Integer def) {
        try {
            return get(strName,Integer.class,def);
        }
        catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    @Override
    public BigInteger getBigInteger(String strName, BigInteger nDefault) {
        try {
            return get(strName,BigInteger.class,nDefault);
        }
        catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }


    @Override
    public Boolean getBoolean(String strName, Boolean def) {
        try {
            return get(strName,Boolean.class,def);
        }
        catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }


    @Override
    public Double getDouble(String strName, Double def) {
        try {
            return get(strName,Double.class,def);
        }
        catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }


    @Override
    public Float getFloat(String strName, Float def) {
        try {
            return get(strName,Float.class,def);
        }
        catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }


    @Override
    public Long getLong(String strName, Long def) {
        try {
            return get(strName,Long.class,def);
        }
        catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }


    @Override
    public String getString(String strName, String strDefault) {
        try {
            String str = get(strName,String.class,"");
            if(StringUtils.isEmpty(str))
                return strDefault;
            return str;
        }
        catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    public <V> V get(String strName,Class<V> clazz,Object defaultValue)
    {
        Object objValue=this.get(strName);
        if(objValue==null) return defaultValue==null?null:(V)defaultValue;
        return TypeUtils.castToJavaBean(objValue,clazz);
    }

    public static <V> V getValue(Object objValue,Class<V> clazz,Object defaultValue)
    {
        if(objValue==null) return defaultValue==null?null:(V)defaultValue;
        return TypeUtils.castToJavaBean(objValue,clazz);
    }


}

